



/*
LISTING 1 DRAW A SIMPLE BITMAP
*/

/*
This function displays a  4-bit bitmap described by the 
BitmapData structure.
It has one restriction.
1. The bitmap must be an even number of pixels wide.

left, top is the location of the upper left dot
of the bitmap on the display.
*/

void imageRender(const BitmapData * imagePtr, int left, int top)
{
    
  /*
  x and y will track the location we are 
  drawing within the bitmap.
  */
  unsigned int x,y;
  /*
  index tracks our location within the array of bytes
  that represent the image. 
  */
  long index;
  /*
  The value of pixel is the color of the dot
  */
  unsigned char pixel;

	/*
	Make sure that the bitmap does not go outside of the display.
	*/
	assert(top + imagePtr->height <= DISPLAY_FULL_HEIGHT);
	assert(left + imagePtr->width <= DISPLAY_FULL_WIDTH);

   
  index = 0;

  for(y=0; y<imagePtr->height; y++)
  {
    /*
    x will step forward 2 at a time, since each 
    byte contains 2 pixels
    */
    for(x=0; x<imagePtr->width; x+=2)
    {   
      /*
      First extract the color of the next pixel
      from the high order nibble
      */
      pixel = imagePtr->raw[index] >> 4;
      /*
      Now place a dot, with the color pixel in
      the next location on the display
      */
      drawDot(x + left, y + top, pixel);
      /*
      First extract the color of the next pixel
      from the high order nibble
      */
      pixel = imagePtr->raw[index] & (0x0F);
      drawDot(x+1 + left, y + top, pixel);
      /*
      index increases once for every two dots drawn,
      since we have two dots-per-pixel.
      */
      index++;
    }
  }
}
        
/*
LISTING 2 DRAW A BITMAP FOR THE YGV CONTROLLER
*/
        
/*
This function displays a  4-bit bitmap described by the 
BitmapData structure. It will display the bitmap by
writing the data directly to RAM that can then be
read by the YGV601B controller.

VRAM_BASE is defined as the address in video-RAM of
the top left pixel of the display.

DISPLAY_BYTE_WIDTH is the number of bytes wide the display is.
For a 4-bits per pixel screen, the DISPLAY_BYTE_WIDTH is
half of the width of the display.

It has two restrictions.
1. The bitmap must be an even number of pixels wide.
2. The left value passed in must be an even number.

Removing the second restriction would make the routine
more general, but a lot slower.

left, top is the location of the upper left dot
of the bitmap on the display.
*/

void imageRenderForYGV(BitmapData * imagePtr, int left, int top)
{
    
  /*
  x and y will track the location we are 
  drawing within the bitmap.
  */
  unsigned int x,y;
  /*
  index tracks our location within the array of bytes
  that represent the image. 
  */
  long index;
  unsigned char *imageOffset;

  /*
  The imageOffset is the location in video RAM where 
  we want to display the top left dot of the bitmap.
  */
  imageOffset = VRAM_BASE + top*DISPLAY_BYTE_WIDTH + left/2;

  /*
  Bitmaps must start on an even boundary to allow a byte for byte
  mapping.
  */
  assert(left%2 ==0);

  /*
  Make sure that the bitmap does not go outside of the display.
  */
  assert(top + imagePtr->height <= DISPLAY_FULL_HEIGHT);
  assert(left + imagePtr->width <= DISPLAY_FULL_WIDTH);


  index = 0;

  for(y=0; y<imagePtr->height; y++)
  {
    /*
    x will step forward 2 at a time, since each 
    byte contains 2 pixels
    */
    for(x=0; x<imagePtr->width; x+=2)
    {   
      /*
      The left hand side of this assignment is the location 
      in video memory where the next one byte (two dots) are 
      to be displayed.
      */
      *(imageOffset + (y*DISPLAY_BYTE_WIDTH) + x/2) =
            imagePtr->raw[index];
      index++;
    }
  }
}

/*
LISTING 3 DISPLAY A STRING
*/
        

/*
This function draws a single character in the color specified.#
*/
void fontDrawMono(const CharInfo *chInfoPtr, 
                    int x, int y, 
                    unsigned char color)
{
  unsigned int i,j;
  unsigned char eightBits;

  /*
  We use index as well as the loop counters because the number
  of bytes per line varies 
  */
  int index=0;

  for (i=0; i< chInfoPtr->height; i++)
  {
    for (j= 0; j< chInfoPtr->width; j++)
    {
      if (j%8 == 0)
      {
        /*
        We are starting a new byte so get the value into eightBits
        */
        eightBits = chInfoPtr->bits[index];
        index++;
      }
      /*
      We plot the pixel if the bit is set. Note that we do not draw
      the unset bits in the background color. We assume that the
      background was cleared if necessary.
      */
      if (eightBits & 0x80)
      {
        /*
        The bit is set so plot the pixel
        */
        drawDot(x+j, y+i, color);

      }

      /*
      Move on to the next pixel in the character.
      */
      eightBits <<= 1;
    }
  }
}
    
/*
x,y position passed in is the start of the baseline.
x is increased as the string is rendered from left to
right, but y stays constant.
*/
void fontRenderString(const char *string, 
                        const CharInfo *font, 
                        int x, int y,
                        unsigned char color)
{
  int yForThisChar;
  const CharInfo *chInfoPtr;
  while (*string)
  {
    chInfoPtr = &font[*string];

    x = x+chInfoPtr->offsetX ;
    yForThisChar = y-chInfoPtr->offsetY - chInfoPtr->height;

    /*
    Draw the character as a monochrome bitmap
    */
    fontDrawMono(chInfoPtr, x, yForThisChar, color);

    x += chInfoPtr->width;

    string++;
  }

}
    


        
  




